from __future__ import unicode_literals
__docformat__ = 'restructuredtext'

from fipy.variables.cellVariable import CellVariable

from fipy.viewers.vtkViewer.vtkViewer import VTKViewer

__all__ = ["VTKCellViewer"]
from future.utils import text_to_native_str
__all__ = [text_to_native_str(n) for n in __all__]

class VTKCellViewer(VTKViewer):
    """Renders `CellVariable` data in VTK format
    """
    def _makeDataSet(self, mesh):
        return mesh.VTKCellDataSet

    @property
    def _data(self):
        return self.dataset.cell_data

    @property
    def _variableClass(self):
        return CellVariable

    def _test(self):
        """
        >>> import os
        >>> from tempfile import mkstemp
        >>> f, fname = mkstemp(".vtk")
        >>> os.close(f)

        >>> try:
        ...     from tvtk.api import tvtk
        ... except ImportError as e:
        ...     from enthought.tvtk.api import tvtk
        ... # doctest: +TVTK

        >>> from fipy import *
        >>> from fipy.viewers.vtkViewer import VTKCellViewer

        >>> m = Grid1D(nx=10)
        >>> x, = m.cellCenters
        >>> v1 = CellVariable(mesh=m, value=x*x, name="x*x")
        >>> v2 = CellVariable(mesh=m, value=x)
        >>> v3 = v1.grad
        >>> v3.name = "v1.grad"
        >>> VTKCellViewer(vars=(v1, v2, v3)).plot(fname) # doctest: +TVTK
        >>> r = tvtk.DataSetReader() # doctest: +TVTK
        >>> r.file_name = fname # doctest: +TVTK
        >>> r.update() # doctest: +TVTK
        >>> c = r.output.cell_data # doctest: +TVTK
        >>> numerix.allclose(c.get_array("x*x").to_array(),
        ...                  v1.value) # doctest: +TVTK, +SERIAL
        True
        >>> numerix.allclose(c.scalars.to_array(),
        ...                  v2.value) # doctest: +TVTK, +SERIAL
        True
        >>> numerix.allclose(c.get_array("v1.grad").to_array().swapaxes(0, 1)[0],
        ...                  v3.value) # doctest: +TVTK, +SERIAL
        True
        >>> r.get_vectors_name_in_file(0) == v3.name  # doctest: +TVTK, +PROCESSOR_0
        True

        >>> m = Grid2D(nx=1, ny=2)
        >>> x, y = m.cellCenters
        >>> v1 = CellVariable(mesh=m, value=x*y, name="x*y")
        >>> v2 = CellVariable(mesh=m, value=x*x) #, name="v2")
        >>> v3 = v1.grad
        >>> v3.name = "v1.grad"
        >>> VTKCellViewer(vars=(v1, v2, v3)).plot(fname) # doctest: +TVTK
        >>> r = tvtk.DataSetReader() # doctest: +TVTK
        >>> r.file_name = fname # doctest: +TVTK
        >>> r.update() # doctest: +TVTK
        >>> c = r.output.cell_data # doctest: +TVTK
        >>> numerix.allclose(c.get_array("x*y").to_array(),
        ...                  v1.value) # doctest: +TVTK, +SERIAL
        True
        >>> numerix.allclose(c.scalars.to_array(),
        ...                  v2.value) # doctest: +TVTK, +SERIAL
        True
        >>> numerix.allclose(c.get_array("v1.grad").to_array().swapaxes(0, 1)[0:2],
        ...                  v3.value) # doctest: +TVTK, +SERIAL
        True
        >>> r.get_vectors_name_in_file(0) == v3.name  # doctest: +TVTK, +PROCESSOR_0
        True

        >>> m = (Grid2D(nx=5, ny=10, dx=0.1, dy=0.1)
        ...      + (Tri2D(nx=5, ny=5, dx=0.1, dy=0.1))
        ...      + ((0.5,), (0.2,)))
        >>> x, y = m.cellCenters
        >>> v1 = CellVariable(mesh=m, value=x*y, name="x*y")
        >>> v2 = CellVariable(mesh=m, value=x*x) #, name="v2")
        >>> v3 = v1.grad
        >>> v3.name = "v1.grad"
        >>> VTKCellViewer(vars=(v1, v2, v3)).plot(fname) # doctest: +TVTK
        >>> r = tvtk.DataSetReader() # doctest: +TVTK
        >>> r.file_name = fname # doctest: +TVTK
        >>> r.update() # doctest: +TVTK
        >>> c = r.output.cell_data # doctest: +TVTK
        >>> numerix.allclose(c.get_array("x*y").to_array(),
        ...                  v1.value) # doctest: +TVTK, +SERIAL
        True
        >>> numerix.allclose(c.scalars.to_array(),
        ...                  v2.value) # doctest: +TVTK, +SERIAL
        True
        >>> numerix.allclose(c.get_array("v1.grad").to_array().swapaxes(0, 1)[0:2],
        ...                  v3.value) # doctest: +TVTK, +SERIAL
        True
        >>> r.get_vectors_name_in_file(0) == v3.name  # doctest: +TVTK, +PROCESSOR_0
        True

        >>> m = Grid3D(nx=2, ny=1, nz=1)
        >>> x, y, z = m.cellCenters
        >>> v1 = CellVariable(mesh=m, value=x*y*z, name="x*y*z")
        >>> v2 = CellVariable(mesh=m, value=x*y*y, name="x*y*y")
        >>> v3 = v1.grad
        >>> v3.name = "v1.grad"
        >>> VTKCellViewer(vars=(v1, v2, v3)).plot(filename=fname) # doctest: +TVTK
        >>> r = tvtk.DataSetReader() # doctest: +TVTK
        >>> r.file_name = fname # doctest: +TVTK
        >>> r.update() # doctest: +TVTK
        >>> c = r.output.cell_data # doctest: +TVTK
        >>> numerix.allclose(c.get_array("x*y*z").to_array(),
        ...                  v1.value) # doctest: +TVTK, +SERIAL
        True
        >>> numerix.allclose(c.scalars.to_array(),
        ...                  v2.value) # doctest: +TVTK, +SERIAL
        True
        >>> numerix.allclose(c.get_array("v1.grad").to_array().swapaxes(0, 1),
        ...                  v3.value) # doctest: +TVTK, +SERIAL
        True
        >>> r.get_scalars_name_in_file(0) == v2.name  # doctest: +TVTK, +PROCESSOR_0
        True
        >>> r.get_vectors_name_in_file(0) == v3.name  # doctest: +TVTK, +PROCESSOR_0
        True

        >>> os.remove(fname)
        """

def _test():
    import fipy.tests.doctestPlus
    return fipy.tests.doctestPlus.testmod()

if __name__ == "__main__":
    _test()





from __future__ import unicode_literals
from builtins import object
import copy

__all__ = ["BlockJacobiSmoother",
           "MultiColorDILUSmoother",
           "MultiColorGSSmoother",
           "MultiColorILUSmoother"]
from future.utils import text_to_native_str
__all__ = [text_to_native_str(n) for n in __all__]

class Smoother(object):
    def __init__(self, smoother_type):
        self.config_dict = {
            "solver": smoother_type,
            "max_iters": 1
        }
    def __call__(self, **kwargs):
        self.config_dict.update(kwargs)
        return copy.copy(self.config_dict)

BlockJacobiSmoother = Smoother("BLOCK_JACOBI")
MultiColorDILUSmoother = Smoother("MULTICOLOR_DILU")
MultiColorGSSmoother = Smoother("MULTICOLOR_GS")
MultiColorILUSmoother = Smoother("MULTICOLOR_ILU")



from __future__ import division
from __future__ import unicode_literals
from builtins import object
__docformat__ = 'restructuredtext'

import os

from fipy import input
from fipy.tools import numerix
from fipy.terms import AbstractBaseClassError
from fipy.terms import SolutionVariableRequiredError

__all__ = ["Term"]
from future.utils import text_to_native_str
__all__ = [text_to_native_str(n) for n in __all__]

class Term(object):
    """
    .. attention:: This class is abstract. Always create one of its subclasses.
    """

    def __init__(self, coeff=1., var=None):
        """
        Create a `Term`.

        Parameters
        ----------
        coeff : float or ~fipy.variables.cellVariable.CellVariable or ~fipy.variables.faceVariable.FaceVariable
            Coefficient for the term.  `FaceVariable` objects are only
            acceptable for diffusion or convection terms.
        """

        if self.__class__ is Term:
            raise AbstractBaseClassError

        self.coeff = coeff
        self.geomCoeff = None
        self._cacheMatrix = False
        self._matrix = None
        self._cacheRHSvector = False
        self._RHSvector = None
        self.var = var

    def _calcVars(self):
        raise NotImplementedError

    def _checkCoeff(self, var):
        raise NotImplementedError

    def copy(self):
        return self.__class__(self.coeff, var=self.var)

    def _buildMatrix(self, var, SparseMatrix, boundaryConditions=(), dt=None, transientGeomCoeff=None, diffusionGeomCoeff=None):
        raise NotImplementedError

    def _buildAndAddMatrices(self, var, SparseMatrix, boundaryConditions=(), dt=None, transientGeomCoeff=None, diffusionGeomCoeff=None, buildExplicitIfOther=False):
        raise NotImplementedError

    def _checkVar(self, var):
        raise NotImplementedError

    def _buildCache(self, matrix, RHSvector):
        if self._cacheMatrix:
            self._matrix = matrix
            self._matrix.cache = True
        else:
            self._matrix = None

        if self._cacheRHSvector:
            self._RHSvector = RHSvector
        else:
            self._RHSvector = None

    def _verifyVar(self, var):
        if var is None:
            if self.var is None:
                raise SolutionVariableRequiredError
            else:
                return self.var
        else:
            return var

    @property
    def _buildExplcitIfOther(self):
        raise NotImplementedError

    def _reshapeIDs(self, var, ids):
        raise NotImplementedError

    def _vectorSize(self, var=None):
        if var is None or var.rank != 1:
            return 1
        else:
            return var.shape[0]

    def _getMatrixClass(self, solver, var):
        if self._vectorSize(var) > 1:
            from fipy.matrices.offsetSparseMatrix import OffsetSparseMatrix
            SparseMatrix =  OffsetSparseMatrix(SparseMatrix=solver._matrixClass,
                                               numberOfVariables=self._vectorSize(var),
                                               numberOfEquations=self._vectorSize(var))
        else:
            SparseMatrix = solver._matrixClass

        return SparseMatrix

    def _prepareLinearSystem(self, var, solver, boundaryConditions, dt):
        solver = self.getDefaultSolver(var, solver)

        var = self._verifyVar(var)
        self._checkVar(var)

        if type(boundaryConditions) not in (type(()), type([])):
            boundaryConditions = (boundaryConditions,)

        for bc in boundaryConditions:
            bc._resetBoundaryConditionApplied()

        if 'FIPY_DISPLAY_MATRIX' in os.environ:
            if not hasattr(self, "_viewer"):
                from fipy.viewers.matplotlibViewer.matplotlibSparseMatrixViewer import MatplotlibSparseMatrixViewer
                Term._viewer = MatplotlibSparseMatrixViewer()

        var, matrix, RHSvector = self._buildAndAddMatrices(var,
                                                           self._getMatrixClass(solver, var),
                                                           boundaryConditions=boundaryConditions,
                                                           dt=dt,
                                                           transientGeomCoeff=self._getTransientGeomCoeff(var),
                                                           diffusionGeomCoeff=self._getDiffusionGeomCoeff(var),
                                                           buildExplicitIfOther=self._buildExplcitIfOther)

        self._buildCache(matrix, RHSvector)

        solver._storeMatrix(var=var, matrix=matrix, RHSvector=RHSvector)

        if 'FIPY_DISPLAY_MATRIX' in os.environ:
            if var is None:
                name = ""
            else:
                if not hasattr(var, "name"):
                    name = ""
                else:
                    name = var.name
            self._viewer.title = r"%s %s" % (name, repr(self))
            from fipy.variables.coupledCellVariable import _CoupledCellVariable
            if isinstance(solver.RHSvector, _CoupledCellVariable):
                RHSvector = solver.RHSvector.globalValue
            else:
                RHSvector = solver.RHSvector
            self._viewer.plot(matrix=solver.matrix, RHSvector=RHSvector)
            from fipy import input
            input()

        return solver

    def solve(self, var=None, solver=None, boundaryConditions=(), dt=None):
        r"""
        Builds and solves the `Term`'s linear system once. This method
        does not return the residual. It should be used when the
        residual is not required.

        Parameters
        ----------
        var : ~fipy.variables.cellVariable.CellVariable
            `Variable` to be solved for.  Provides the initial condition,
            the old value and holds the solution on completion.
        solver : ~fipy.solvers.solver.Solver
            Iterative solver to be used to solve the linear system of
            equations.  The default sovler depends on the solver package
            selected.
        boundaryConditions : :obj:`tuple` of :obj:`~fipy.boundaryConditions.boundaryCondition.BoundaryCondition`
        dt : float
            Timestep size.
        """

        solver = self._prepareLinearSystem(var, solver, boundaryConditions, dt)

        solver._solve()

    def sweep(self, var=None, solver=None, boundaryConditions=(), dt=None, underRelaxation=None, residualFn=None, cacheResidual=False, cacheError=False):
        r"""
        Builds and solves the `Term`'s linear system once. This method
        also recalculates and returns the residual as well as applying
        under-relaxation.

        Parameters
        ----------
        var : ~fipy.variables.cellVariable.CellVariable
            `Variable` to be solved for.  Provides the initial condition,
            the old value and holds the solution on completion.
        solver : ~fipy.solvers.solver.Solver
            Iterative solver to be used to solve the linear system of
            equations.  The default sovler depends on the solver package
            selected.
        boundaryConditions : :obj:`tuple` of :obj:`~fipy.boundaryConditions.boundaryCondition.BoundaryCondition`
        dt : float
            Timestep size.
        underRelaxation : float
            Usually a value between `0` and `1` or `None` in the case of no
            under-relaxation
        residualFn : function
            Takes `var`, `matrix`, and `RHSvector` arguments, used to
            customize the residual calculation.
        cacheResidual : bool
            If `True`, calculate and store the residual vector
            :math:`\vec{r}=\mathsf{L}\vec{x} - \vec{b}` in the
            `residualVector` member of `Term`
        cacheError : bool
            If `True`, use the residual vector :math:`\vec{r}` to solve
            :math:`\mathsf{L}\vec{e}=\vec{r}` for the error vector
            :math:`\vec{e}` and store it in the `errorVector` member of
            `Term`
        
        Returns
        -------
        residual : ~fipy.variables.cellVariable.CellVariable
            The residual vector :math:`\vec{r}=\mathsf{L}\vec{x} - \vec{b}`
        """
        solver = self._prepareLinearSystem(var=var, solver=solver, boundaryConditions=boundaryConditions, dt=dt)
        solver._applyUnderRelaxation(underRelaxation=underRelaxation)
        residual = solver._calcResidual(residualFn=residualFn)

        if cacheResidual or cacheError:
            self.residualVector = solver._calcResidualVector(residualFn=residualFn)

        if cacheError:
            self.errorVector = solver.var.copy()
            var_tmp = solver.var
            RHS_tmp = solver.RHSvector
            solver._storeMatrix(var=self.errorVector, matrix=solver.matrix, RHSvector=self.residualVector)
            solver._solve()
            solver._storeMatrix(var=var_tmp, matrix=solver.matrix, RHSvector=RHS_tmp)

        if not cacheResidual:
            self.residualVector = None

        solver._solve()

        return residual

    def justResidualVector(self, var=None, solver=None, boundaryConditions=(), dt=None, underRelaxation=None, residualFn=None):
        r"""Builds the `Term`'s linear system once.

        This method also recalculates and returns the residual as well as
        applying under-relaxation.

        `justResidualVector` returns the overlapping local value in parallel (not the non-overlapping value).

        >>> from fipy import *
        >>> m = Grid1D(nx=10)
        >>> v = CellVariable(mesh=m)
        >>> len(numerix.asarray(DiffusionTerm().justResidualVector(v))) == m.numberOfCells
        True

        Parameters
        ----------
        var : ~fipy.variables.cellVariable.CellVariable
            `Variable` to be solved for.  Provides the initial condition,
            the old value and holds the solution on completion.
        solver : ~fipy.solvers.solver.Solver
            Iterative solver to be used to solve the linear system of
            equations.  The default sovler depends on the solver package
            selected.
        boundaryConditions : :obj:`tuple` of :obj:`~fipy.boundaryConditions.boundaryCondition.BoundaryCondition`
        dt : float
            Timestep size.
        underRelaxation : float
            Usually a value between `0` and `1` or `None` in the case of no
            under-relaxation
        residualFn : function
            Takes `var`, `matrix`, and `RHSvector` arguments, used to
            customize the residual calculation.

        Returns
        -------
        residual : ~fipy.variables.cellVariable.CellVariable
            The residual vector :math:`\vec{r}=\mathsf{L}\vec{x} - \vec{b}`
        """
        solver = self._prepareLinearSystem(var, solver, boundaryConditions, dt)
        solver._applyUnderRelaxation(underRelaxation)

        return solver._calcResidualVector(residualFn=residualFn)

    def residualVectorAndNorm(self, var=None, solver=None, boundaryConditions=(), dt=None, underRelaxation=None, residualFn=None):
        r"""Builds the `Term`'s linear system once.

        This method also recalculates and returns the residual as well as
        applying under-relaxation.

        Parameters
        ----------
        var : ~fipy.variables.cellVariable.CellVariable
            `Variable` to be solved for.  Provides the initial condition,
            the old value and holds the solution on completion.
        solver : ~fipy.solvers.solver.Solver
            Iterative solver to be used to solve the linear system of
            equations.  The default sovler depends on the solver package
            selected.
        boundaryConditions : :obj:`tuple` of :obj:`~fipy.boundaryConditions.boundaryCondition.BoundaryCondition`
        dt : float
            Timestep size.
        underRelaxation : float
            Usually a value between `0` and `1` or `None` in the case of no
            under-relaxation
        residualFn : function
            Takes `var`, `matrix`, and `RHSvector` arguments, used to
            customize the residual calculation.
            
        Returns
        -------
        residual : ~fipy.variables.cellVariable.CellVariable
            The residual vector :math:`\vec{r}=\mathsf{L}\vec{x} - \vec{b}`
        norm : float
            The L2 norm of `residual`, :math:`\|\vec{r}\|_2`
        """
        vector = self.justResidualVector(var=var, solver=solver, boundaryConditions=boundaryConditions, dt=dt,
                                         underRelaxation=underRelaxation, residualFn=residualFn)

        L2norm = numerix.L2norm(vector)

        return vector, L2norm

    def justErrorVector(self, var=None, solver=None, boundaryConditions=(), dt=1., underRelaxation=None, residualFn=None):
        r"""Builds the `Term`'s linear system once.

        This method also recalculates and returns the error as well as
        applying under-relaxation.

        `justErrorVector` returns the overlapping local value in parallel (not the non-overlapping value).

        >>> from fipy.solvers import DummySolver
        >>> from fipy import *
        >>> m = Grid1D(nx=10)
        >>> v = CellVariable(mesh=m)
        >>> len(DiffusionTerm().justErrorVector(v, solver=DummySolver())) == m.numberOfCells
        True

        Parameters
        ----------
        var : ~fipy.variables.cellVariable.CellVariable
            `Variable` to be solved for.  Provides the initial condition,
            the old value and holds the solution on completion.
        solver : ~fipy.solvers.solver.Solver
            Iterative solver to be used to solve the linear system of
            equations.  The default sovler depends on the solver package
            selected.
        boundaryConditions : :obj:`tuple` of :obj:`~fipy.boundaryConditions.boundaryCondition.BoundaryCondition`
        dt : float
            Timestep size.
        underRelaxation : float
            Usually a value between `0` and `1` or `None` in the case of no
            under-relaxation
        residualFn : function
            Takes `var`, `matrix`, and `RHSvector` arguments, used to
            customize the residual calculation.

        Returns
        -------
        error : ~fipy.variables.cellVariable.CellVariable
            The residual vector :math:`\vec{e}=\mathsf{L}\vec{x}_\text{old} - \vec{b}`
        """
        solver = self._prepareLinearSystem(var, solver, boundaryConditions, dt)
        solver._applyUnderRelaxation(underRelaxation)
        residualVector = solver._calcResidualVector(residualFn=residualFn)

        errorVector = solver.var.copy()
        solver._storeMatrix(var=errorVector, matrix=solver.matrix, RHSvector=residualVector)
        solver._solve()

        return errorVector

    def cacheMatrix(self):
        r"""
        Informs `solve()` and `sweep()` to cache their matrix so
        that `matrix` can return the matrix.
        """
        self._cacheMatrix = True

    @property
    def matrix(self):
        r"""
        Return the matrix calculated in `solve()` or `sweep()`. The
        `cacheMatrix()` method should be called before `solve()` or
        `sweep()` to cache the matrix.
        """
        if not self._cacheMatrix:
            import warnings
            warnings.warn("""cacheMatrix() should be called followed by sweep() or solve()
                          to calculate the matrix. None will be returned on this occasion.""",
                          UserWarning, stacklevel=2)
            self.cacheMatrix()

        return self._matrix

    def cacheRHSvector(self):
        r"""
        Informs `solve()` and `sweep()` to cache their right hand side
        vector so that `getRHSvector()` can return it.
        """
        self._cacheRHSvector = True

    @property
    def RHSvector(self):
        r"""
        Return the RHS vector calculated in `solve()` or `sweep()`. The
        `cacheRHSvector()` method should be called before `solve()` or
        `sweep()` to cache the vector.
        """
        if not self._cacheRHSvector:
            import warnings
            warnings.warn("""getRHSvector should be called after cacheRHSvector() and either sweep() or solve()
                          to calculate the RHS vector. None will be returned on this occasion.""",
                          UserWarning, stacklevel=2)
            self.cacheRHSvector()

        return self._RHSvector

    def _getDefaultSolver(self, var, solver, *args, **kwargs):
        return NotImplementedError

    def getDefaultSolver(self, var=None, solver=None, *args, **kwargs):
        from fipy.solvers import DefaultSolver
        return solver or self._getDefaultSolver(var, solver, *args, **kwargs) or DefaultSolver(*args, **kwargs)

    def __add__(self, other):
        if isinstance(other, (int, float)) and other == 0:
            return self
        else:
            from fipy.terms.binaryTerm import _BinaryTerm
            return _BinaryTerm(self, other)

    __radd__ = __add__

    def __neg__(self):
        raise NotImplementedError

    def __pos__(self):
        return self

    def __sub__(self, other):
        return self + (-other)

    def __rsub__(self, other):
        return other + (-self)

    def __eq__(self, other):
        return self - other

    __hash__ = object.__hash__

    def __mul__(self, other):
        raise NotImplementedError

    __rmul__ = __mul__

    def __truediv__(self, other):
        return (1 / other) * self

    __div__ = __truediv__

    def __and__(self, other):
        if isinstance(other, Term):
            from fipy.terms.coupledBinaryTerm import _CoupledBinaryTerm
            return _CoupledBinaryTerm(self, other)
        elif other == 0:
            return self
        else:
            raise Exception("Can only couple Term objects.")

    __rand__ = __and__

    def __repr__(self):
        raise NotImplementedError

    def _calcGeomCoeff(self, var):
        raise NotImplementedError

    def _getGeomCoeff(self, var):
        if self.geomCoeff is None:
            self.geomCoeff = self._calcGeomCoeff(var)
            if self.geomCoeff is not None:
                self.geomCoeff.dontCacheMe()

        return self.geomCoeff

    def _getWeight(self, var, transientGeomCoeff=None, diffusionGeomCoeff=None):
        raise NotImplementedError

    def _getDiagonalSign(self, transientGeomCoeff=None, diffusionGeomCoeff=None):
        raise NotImplementedError

    def _getDiffusionGeomCoeff(self, var):
        return None

    def _getTransientGeomCoeff(self, var):
        return None

    def _getNormals(self, mesh):
        raise NotImplementedError

    def _getOldAdjacentValues(self, oldArray, id1, id2, dt):
        raise NotImplementedError

    def _getDifferences(self, adjacentValues, cellValues, oldArray, cellToCellIDs, mesh):
        raise NotImplementedError

    def _alpha(self, P):
        raise NotImplementedError

    def _checkDt(self, dt):
        raise NotImplementedError

    def _test(self):
        """
        Test stuff.

        >>> from fipy import *
        >>> L = 1.
        >>> nx = 100
        >>> m = Grid1D(nx=nx, dx=L / nx)
        >>> v = CellVariable(mesh=m, value=1.)
        >>> eqn = DiffusionTerm() - v

        >>> v.constrain(0.,  m.facesLeft)
        >>> v.constrain(1.,  m.facesRight)

        >>> res = 1.
        >>> sweep = 0
        >>> while res > 1e-8 and sweep < 100:
        ...     res = eqn.sweep(v)
        ...     sweep += 1
        >>> x = m.cellCenters[0]
        >>> answer = (numerix.exp(x) - numerix.exp(-x)) / (numerix.exp(L) - numerix.exp(-L))
        >>> print(numerix.allclose(v, answer, rtol=2e-5))
        True

        >>> v.setValue(0.)
        >>> eqn = DiffusionTerm(0.2) * 5. - 5. * ImplicitSourceTerm(0.2)
        >>> eqn.solve(v)
        >>> print(numerix.allclose(v, answer, rtol=2e-5))
        True

        >>> v.setValue(0.)
        >>> eqn = 2. * (DiffusionTerm(1.) - ImplicitSourceTerm(.5)) - DiffusionTerm(1.)
        >>> eqn.solve(v)
        >>> print(numerix.allclose(v, answer, rtol=2e-5))
        True

        >>> from fipy import Grid1D, CellVariable, DiffusionTerm, TransientTerm
        >>> mesh = Grid1D(nx=3)
        >>> A = CellVariable(mesh=mesh, name="A", value=[1., 2., 3.])
        >>> B = CellVariable(mesh=mesh, name="B", value=[3., 4., 5.])
        >>> C = CellVariable(mesh=mesh, name="C")

        >>> eq = DiffusionTerm(coeff=1.)
        >>> solver = eq._prepareLinearSystem(var=None, solver=None, boundaryConditions=(), dt=1.) # doctest: +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
        ...
        SolutionVariableRequiredError: The solution variable needs to be specified.
        >>> solver = eq._prepareLinearSystem(var=A, solver=None, boundaryConditions=(), dt=1.)
        >>> numpyMatrix = solver.matrix.numpyArray
        >>> print(numerix.allequal(numpyMatrix, [[-1, 1, 0],
        ...                                      [ 1, -2, 1],
        ...                                      [ 0, 1, -1]]))
        ... # doctest: +PROCESSOR_0
        True
        >>> print(numerix.allequal(solver.RHSvector, [0, 0, 0])) # doctest: +PROCESSOR_0
        True

        >>> eq = DiffusionTerm(coeff=1., var=A)
        >>> solver = eq._prepareLinearSystem(var=None, solver=None, boundaryConditions=(), dt=1.)
        >>> numpyMatrix = solver.matrix.numpyArray
        >>> print(numerix.allequal(numpyMatrix, [[-1, 1, 0],
        ...                                      [ 1, -2, 1],
        ...                                      [ 0, 1, -1]]))
        ... # doctest: +PROCESSOR_0
        True
        >>> print(numerix.allequal(solver.RHSvector, [0, 0, 0])) # doctest: +PROCESSOR_0
        True
        >>> solver = eq._prepareLinearSystem(var=B, solver=None, boundaryConditions=(), dt=1.)
        >>> numpyMatrix = solver.matrix.numpyArray
        >>> print(numerix.allequal(numpyMatrix, [[ 0, 0, 0],
        ...                                      [ 0, 0, 0],
        ...                                      [ 0, 0, 0]]))
        ... # doctest: +PROCESSOR_0
        True
        >>> print(numerix.allequal(solver.RHSvector, [0, 0, 0])) # doctest: +PROCESSOR_0
        True

        >>> eq = TransientTerm(coeff=1.) == DiffusionTerm(coeff=1.)
        >>> solver = eq._prepareLinearSystem(var=None, solver=None, boundaryConditions=(), dt=1.) # doctest: +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
        ...
        SolutionVariableRequiredError: The solution variable needs to be specified.
        >>> solver = eq._prepareLinearSystem(var=A, solver=None, boundaryConditions=(), dt=1.)
        >>> numpyMatrix = solver.matrix.numpyArray
        >>> print(numerix.allequal(numpyMatrix, [[ 2, -1, 0],
        ...                                      [-1, 3, -1],
        ...                                      [ 0, -1, 2]]))
        ... # doctest: +PROCESSOR_0
        True
        >>> print(numerix.allequal(solver.RHSvector, [1, 2, 3])) # doctest: +PROCESSOR_0
        True

        >>> eq = TransientTerm(coeff=1., var=A) == DiffusionTerm(coeff=1., var=A)
        >>> solver = eq._prepareLinearSystem(var=None, solver=None, boundaryConditions=(), dt=1.)
        >>> numpyMatrix = solver.matrix.numpyArray
        >>> print(numerix.allequal(numpyMatrix, [[ 2, -1, 0],
        ...                                      [-1, 3, -1],
        ...                                      [ 0, -1, 2]]))
        ... # doctest: +PROCESSOR_0
        True
        >>> print(numerix.allequal(solver.RHSvector, [1, 2, 3])) # doctest: +PROCESSOR_0
        True
        >>> solver = eq._prepareLinearSystem(var=A, solver=None, boundaryConditions=(), dt=1.)
        >>> numpyMatrix = solver.matrix.numpyArray
        >>> print(numerix.allequal(numpyMatrix, [[ 2, -1, 0],
        ...                                      [-1, 3, -1],
        ...                                      [ 0, -1, 2]]))
        ... # doctest: +PROCESSOR_0
        True
        >>> print(numerix.allequal(solver.RHSvector, [1, 2, 3])) # doctest: +PROCESSOR_0
        True
        >>> solver = eq._prepareLinearSystem(var=B, solver=None, boundaryConditions=(), dt=1.)
        >>> numpyMatrix = solver.matrix.numpyArray
        >>> print(numerix.allequal(numpyMatrix, [[ 0, 0, 0],
        ...                                      [ 0, 0, 0],
        ...                                      [ 0, 0, 0]]))
        ... # doctest: +PROCESSOR_0
        True
        >>> print(numerix.allequal(solver.RHSvector, [1, 0, -1])) # doctest: +PROCESSOR_0
        True

        >>> eq = TransientTerm(coeff=1., var=A) == DiffusionTerm(coeff=1., var=B)
        >>> print(eq)
        (TransientTerm(coeff=1.0, var=A) + DiffusionTerm(coeff=[-1.0], var=B))
        >>> A in set(eq._vars) and B in set(eq._vars) ## _getVars() is unordered for _BinaryTerm's.
        True
        >>> print((eq.term, eq.other))
        (TransientTerm(coeff=1.0, var=A), DiffusionTerm(coeff=[-1.0], var=B))
        >>> res = eq.justResidualVector(boundaryConditions=(), dt=1.)
        >>> print(numerix.allequal(res, [-1, 0, 1]))  # doctest: +PROCESSOR_0
        True
        >>> solver = eq._prepareLinearSystem(var=A, solver=None, boundaryConditions=(), dt=1.)
        >>> numpyMatrix = solver.matrix.numpyArray
        >>> print(numerix.allequal(numpyMatrix, [[1, 0, 0],
        ...                                      [0, 1, 0],
        ...                                      [0, 0, 1]]))
        ... # doctest: +PROCESSOR_0
        True
        >>> print(numerix.allequal(solver.RHSvector, [2, 2, 2])) # doctest: +PROCESSOR_0
        True
        >>> solver = eq._prepareLinearSystem(var=B, solver=None, boundaryConditions=(), dt=1.)
        >>> numpyMatrix = solver.matrix.numpyArray
        >>> print(numerix.allequal(numpyMatrix, [[ 1, -1, 0],
        ...                                      [-1, 2, -1],
        ...                                      [ 0, -1, 1]]))
        ... # doctest: +PROCESSOR_0
        True
        >>> print(numerix.allequal(solver.RHSvector, [0, 0, 0])) # doctest: +PROCESSOR_0
        True
        >>> solver = eq._prepareLinearSystem(var=C, solver=None, boundaryConditions=(), dt=1.)
        >>> numpyMatrix = solver.matrix.numpyArray
        >>> print(numerix.allequal(numpyMatrix, [[ 0, 0, 0],
        ...                                      [ 0, 0, 0],
        ...                                      [ 0, 0, 0]]))
        ... # doctest: +PROCESSOR_0
        True
        >>> print(numerix.allequal(solver.RHSvector, [1, 0, -1])) # doctest: +PROCESSOR_0
        True

        >>> eq = TransientTerm(coeff=1.) == DiffusionTerm(coeff=1., var=B) + 10.  # doctest: +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
            ...
        ExplicitVariableError: Terms with explicit Variables cannot mix with Terms with implicit Variables.
        >>> eq = DiffusionTerm(coeff=1., var=B) + 10. == 0
        >>> print(eq)
        (DiffusionTerm(coeff=[1.0], var=B) + 10.0)
        >>> print(eq._vars)
        [B]
        >>> print((eq.term, eq.other))
        (DiffusionTerm(coeff=[1.0], var=B), 10.0)
        >>> solver = eq._prepareLinearSystem(var=B, solver=None, boundaryConditions=(), dt=1.)
        >>> numpyMatrix = solver.matrix.numpyArray
        >>> print(numerix.allequal(numpyMatrix, [[-1, 1, 0],
        ...                                      [1, -2, 1],
        ...                                      [0, 1, -1]]))
        ... # doctest: +PROCESSOR_0
        True
        >>> print(numerix.allequal(solver.RHSvector, [-10, -10, -10]))  # doctest: +PROCESSOR_0
        True

        >>> from fipy.solvers import DummySolver
        >>> eq.solve(var=B, solver=DummySolver())

        >>> m = Grid1D(nx=2)
        >>> A = CellVariable(mesh=m, name='A')
        >>> B = CellVariable(mesh=m, name='B')
        >>> C = CellVariable(mesh=m, name='C')
        >>> DiffusionTerm().solve() # doctest: +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
            ...
        SolutionVariableRequiredError: The solution variable needs to be specified.
        >>> DiffusionTerm().solve(A, solver=DummySolver())
        >>> DiffusionTerm(var=A).solve(A, solver=DummySolver())
        >>> (DiffusionTerm(var=A) + DiffusionTerm()) # doctest: +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
            ...
        ExplicitVariableError: Terms with explicit Variables cannot mix with Terms with implicit Variables.
        >>> (DiffusionTerm(var=A) + DiffusionTerm(var=B)).solve(solver=DummySolver())
        >>> (DiffusionTerm(var=A) + DiffusionTerm(var=B)).solve(A, solver=DummySolver())
        >>> DiffusionTerm() & DiffusionTerm() # doctest: +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
            ...
        SolutionVariableNumberError: Different number of solution variables and equations.
        >>> DiffusionTerm(var=A) & DiffusionTerm() # doctest: +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
            ...
        ExplicitVariableError: Terms with explicit Variables cannot mix with Terms with implicit Variables.

        >>> A = CellVariable(mesh=m, name='A', value=1)
        >>> B = CellVariable(mesh=m, name='B')
        >>> C = CellVariable(mesh=m, name='C')

        >>> eq = (DiffusionTerm(coeff=1., var=A)) & (DiffusionTerm(coeff=2., var=B))
        >>> eq.cacheMatrix()
        >>> eq.cacheRHSvector()
        >>> eq.solve(solver=DummySolver())
        >>> numpyMatrix = eq.matrix.numpyArray
        >>> print(numerix.allequal(numpyMatrix, [[-1, 1, 0, 0],
        ...                                      [1, -1, 0, 0],
        ...                                      [0, 0, -2, 2],
        ...                                      [0, 0, 2, -2]]))
        ... # doctest: +PROCESSOR_0
        True
        >>> print(eq.RHSvector.globalValue)
        [ 0.  0.  0.  0.]
        >>> print(eq._vars)
        [A, B]
        >>> solver = eq._prepareLinearSystem(var=None, solver=None, boundaryConditions=(), dt=1.)
        >>> numpyMatrix = eq.matrix.numpyArray
        >>> print(numerix.allequal(numpyMatrix, [[-1, 1, 0, 0],
        ...                                      [1, -1, 0, 0],
        ...                                      [0, 0, -2, 2],
        ...                                      [0, 0, 2, -2]]))
        ... # doctest: +PROCESSOR_0
        True
        >>> print(eq.RHSvector.globalValue)
        [ 0.  0.  0.  0.]
        >>> solver = eq._prepareLinearSystem(var=A, solver=None, boundaryConditions=(), dt=1.) # doctest: +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
            ...
        SolutionVariableNumberError: The solution variable should not be specified.
        >>> solver = eq._prepareLinearSystem(var=C, solver=None, boundaryConditions=(), dt=1.) # doctest: +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
            ...
        SolutionVariableNumberError: The solution variable should not be specified.

        >>> DiffusionTerm(var=A) & DiffusionTerm(var=A) # doctest: +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
            ...
        SolutionVariableNumberError: Different number of solution variables and equations.
        >>> DiffusionTerm() & DiffusionTerm() # doctest: +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
            ...
        SolutionVariableNumberError: Different number of solution variables and equations.
        >>> (DiffusionTerm(var=A) & DiffusionTerm(var=B)).solve(A) # doctest: +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
            ...
        SolutionVariableNumberError: The solution variable should not be specified.
        >>> DiffusionTerm(var=A) & DiffusionTerm(var=B) & DiffusionTerm(var=B) # doctest: +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
            ...
        SolutionVariableNumberError: Different number of solution variables and equations.
        >>> (DiffusionTerm(var=A) & DiffusionTerm(var=B) & DiffusionTerm(var=C)).solve(solver=DummySolver())
        >>> (DiffusionTerm(var=A) & DiffusionTerm(var=B) & DiffusionTerm(var=C)).solve(A) # doctest: +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
            ...
        SolutionVariableNumberError: The solution variable should not be specified.
        >>> (DiffusionTerm(var=A) & (DiffusionTerm(var=B) + DiffusionTerm(var=C))).solve(A) # doctest: +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
            ...
        SolutionVariableNumberError: The solution variable should not be specified.
        >>> eq = (DiffusionTerm(coeff=1., var=A) + DiffusionTerm(coeff=2., var=B)) & (DiffusionTerm(coeff=2., var=B) + DiffusionTerm(coeff=3., var=C)) & (DiffusionTerm(coeff=3., var=C) + DiffusionTerm(coeff=1., var=A))
        >>> eq.cacheMatrix()
        >>> eq.cacheRHSvector()
        >>> eq.solve(solver=DummySolver())
        >>> numpyMatrix = eq.matrix.numpyArray
        >>> print(numerix.allequal(numpyMatrix, [[-1, 1, -2, 2, 0, 0],
        ...                                      [1, -1, 2, -2, 0, 0],
        ...                                      [0, 0, -2, 2, -3, 3],
        ...                                      [0, 0, 2, -2, 3, -3],
        ...                                      [-1, 1, 0, 0, -3, 3],
        ...                                      [1, -1, 0, 0, 3, -3]]))
        ... # doctest: +PROCESSOR_0
        True
        >>> print(eq.RHSvector.globalValue)
        [ 0.  0.  0.  0.  0.  0.]
        >>> print(eq._vars)
        [A, B, C]
        >>> eq = DiffusionTerm(var=A)
        >>> print((0 & eq) is eq)
        True
        >>> eq = 0
        >>> eq &= DiffusionTerm(coeff=1., var=A)
        >>> eq &= DiffusionTerm(coeff=2., var=B)
        >>> eq.cacheMatrix()
        >>> eq.cacheRHSvector()
        >>> eq.solve(solver=DummySolver())
        >>> numpyMatrix = eq.matrix.numpyArray
        >>> print(numerix.allequal(numpyMatrix, [[-1, 1, 0, 0],
        ...                                      [1, -1, 0, 0],
        ...                                      [0, 0, -2, 2],
        ...                                      [0, 0, 2, -2]]))
        ... # doctest: +PROCESSOR_0
        True

        Test for ticket:658, the test should run without an error.

        >>> m = Grid1D(nx=3)
        >>> v = CellVariable(mesh=m, elementshape=(2,))
        >>> v.constrain([[0], [1]], m.facesLeft)
        >>> v.constrain([[1], [0]], m.facesRight)
        >>> eqn = TransientTerm() == DiffusionTerm([[[0.01, -1], [1, 0.01]]])
        >>> res = eqn.sweep(var=v, dt=1.)

        """

class __Term(Term):
    """
    Dummy subclass for tests
    """
    pass

def _test():
    import fipy.tests.doctestPlus
    return fipy.tests.doctestPlus.testmod()

if __name__ == "__main__":
    _test()





"""Generic mesh class using numerix to do the calculations

    Meshes contain cells, faces, and vertices.

    This is built for a non-mixed element mesh.
"""
from __future__ import unicode_literals

from fipy.tools import numerix
from fipy.tools.numerix import MA
from fipy.tools import serialComm

from fipy.meshes.mesh import Mesh
from fipy.meshes.representations.meshRepresentation import _MeshRepresentation
from fipy.meshes.topologies.meshTopology import _Mesh1DTopology

__all__ = ["Mesh1D"]
from future.utils import text_to_native_str
__all__ = [text_to_native_str(n) for n in __all__]

class Mesh1D(Mesh):
    def __init__(self, vertexCoords, faceVertexIDs, cellFaceIDs, communicator=serialComm, _RepresentationClass=_MeshRepresentation, _TopologyClass=_Mesh1DTopology):
        super(Mesh1D, self).__init__(vertexCoords=vertexCoords, faceVertexIDs=faceVertexIDs, cellFaceIDs=cellFaceIDs, communicator=communicator,
                                     _RepresentationClass=_RepresentationClass, _TopologyClass=_TopologyClass)

    def _calcScaleArea(self):
        return 1.

    def _calcScaleVolume(self):
        return self.scale['length']

    def _calcFaceAreas(self):
        return numerix.ones(self.numberOfFaces, 'd')

    def _calcFaceNormals(self):
        faceNormals = numerix.array((numerix.ones(self.numberOfFaces, 'd'),))
        # The left-most face has neighboring cells None and the left-most cell.
        # We must reverse the normal to make fluxes work correctly.
        if self.numberOfFaces > 0:
            faceNormals[..., 0] = -faceNormals[..., 0]
        return faceNormals

    def _calcFaceTangents(self):
        faceTangents1 = numerix.zeros(self.numberOfFaces, 'd')[numerix.NewAxis, ...]
        faceTangents2 = numerix.zeros(self.numberOfFaces, 'd')[numerix.NewAxis, ...]
        return faceTangents1, faceTangents2

    def _translate(self, vector):
        newCoords = self.vertexCoords + vector
        newmesh = Mesh1D(newCoords, numerix.array(self.faceVertexIDs), numerix.array(self.cellFaceIDs))
        return newmesh

    def __mul__(self, factor):
        newCoords = self.vertexCoords * factor
        newmesh = Mesh1D(newCoords, numerix.array(self.faceVertexIDs), numerix.array(self.cellFaceIDs))
        return newmesh

    @property
    def _VTKCellType(self):
        try:
            from tvtk.api import tvtk
        except ImportError as e:
            from enthought.tvtk.api import tvtk
        return tvtk.Line().cell_type



from __future__ import unicode_literals
__all__ = []

from fipy.variables.faceVariable import FaceVariable
from fipy.tools import numerix

class _CellToFaceVariable(FaceVariable):
    def __init__(self, var):
        FaceVariable.__init__(self, mesh=var.mesh, elementshape=var.shape[:-1])
        self.var = self._requires(var)

    def _calcValue(self):
        alpha = self.mesh._faceToCellDistanceRatio
        id1, id2 = self.mesh._adjacentCellIDs

        return self._calcValue_(alpha=alpha, id1=id1, id2=id2)

    def release(self, constraint):
        """Remove `constraint` from `self`

        >>> from fipy import *
        >>> m = Grid1D(nx=3)
        >>> v = CellVariable(mesh=m, value=m.cellCenters[0])
        >>> c0 = Constraint(0., where=m.facesLeft)
        >>> v.constrain(c0)
        >>> c1 = Constraint(3., where=m.facesRight)
        >>> v.faceValue.constrain(c1)
        >>> print(v.faceValue)
        [ 0.  1.  2.  3.]
        >>> v.faceValue.release(constraint=c0)
        >>> print(v.faceValue)
        [ 0.5  1.   2.   3. ]
        >>> v.faceValue.release(constraint=c1)
        >>> print(v.faceValue)
        [ 0.5  1.   2.   2.5]
        """
        try:
            super(FaceVariable, self).constraints.remove(constraint)
        except ValueError:
            self.var.release(constraint=constraint)

    @property
    def constraints(self):
        if hasattr(self.var, "faceConstraints"):
            faceConstraints = self.var.faceConstraints
        else:
            faceConstraints = []

        return super(_CellToFaceVariable, self).constraints + faceConstraints

    @property
    def constraintMask(self):
        if hasattr(self, '_constraintMask'):
            if hasattr(self.var, 'faceConstraints'):
                for constraint in self.var.faceConstraints:
                    self._constraintMask._requires(constraint.where)

        return super(_CellToFaceVariable, self).constraintMask

    def __getstate__(self):
        return dict(var=self.var)

    def __setstate__(self, dict):
        self.__init__(**dict)

def _test():
    import fipy.tests.doctestPlus
    return fipy.tests.doctestPlus.testmod()

if __name__ == "__main__":
    _test()




from __future__ import division
from __future__ import unicode_literals
__docformat__ = 'restructuredtext'

from fipy.variables.cellToFaceVariable import _CellToFaceVariable
from fipy.tools.numerix import exp, take, where

__all__ = ["ScharfetterGummelFaceVariable"]
from future.utils import text_to_native_str
__all__ = [text_to_native_str(n) for n in __all__]

class ScharfetterGummelFaceVariable(_CellToFaceVariable):
    def __init__(self, var, boundaryConditions=()):
        _CellToFaceVariable.__init__(self, var)
        self.bcs = boundaryConditions

    def _calcValue_(self, alpha, id1, id2):
        cell1 = take(self.var, id1)
        cell2 = take(self.var, id2)
        delta = cell1 - cell2

        eps = 1e-14
        value = where(abs(delta) < eps, 1. / exp(delta), 0.)
        delta = where(abs(delta) < eps, eps, delta)
        value = where((abs(delta) > eps) & (delta < 100),
                      delta / (exp(delta) - 1), value)

        value *= exp(cell1)

        for bc in self.bcs:
            if isinstance(bc, FixedValue):
                value[bc.faces.value] = bc._value

        return value



from __future__ import unicode_literals
from builtins import range
__docformat__ = 'restructuredtext'

__all__ = []

import scipy.sparse as sp
from fipy.tools import numerix

from fipy.matrices.sparseMatrix import _SparseMatrix

class _ScipyMatrix(_SparseMatrix):

    """class wrapper for a scipy sparse matrix.

    `_ScipyMatrix` is always `NxN`.
    Allows basic python operations __add__, __sub__ etc.
    Facilitate matrix populating in an easy way.
    """

    def __init__(self, matrix):
        """Creates a `_ScipyMatrix`.

        Parameters
        ----------
        matrix : ~scipy.sparse.csr_matrix
            The internal SciPy matrix
        """
        self.matrix = matrix

        super(_ScipyMatrix, self).__init__()

    def copy(self):
        return _ScipyMatrix(matrix=self.matrix.copy())

    def __getitem__(self, index):
        m = self.matrix[index]
        if isinstance(m, (type(0), type(0.))):
            return m
        else:
            return _ScipyMatrix(matrix=m)

    def __iadd__(self, other):
        return self._iadd(other)

    def _iadd(self, other, sign=1):
        if hasattr(other, "matrix"):
            self.matrix = self.matrix + (sign * other.matrix)
        elif isinstance(other, (float, int)):
            fillVec = numerix.repeat(other, self.matrix.nnz)

            self.matrix = self.matrix \
                          + sp.csr_matrix((fillVec, self.matrix.nonzero()),
                                          self.matrix.shape)
        else:
            self.matrix = self.matrix + (sign * other)

        return self

    def __add__(self, other):
        """
        Add two sparse matrices

            >>> L = _ScipyMatrixFromShape(rows=3, cols=3)
            >>> L.put([3., 10., numerix.pi, 2.5], [0, 0, 1, 2], [2, 1, 1, 0])
            >>> print(L + _ScipyIdentityMatrix(size=3))
             1.000000  10.000000   3.000000  
                ---     4.141593      ---    
             2.500000      ---     1.000000  

            >>> print(L + 0)
                ---    10.000000   3.000000  
                ---     3.141593      ---    
             2.500000      ---        ---    

            >>> print(L + 3)
            Traceback (most recent call last):
            ...
            AttributeError: 'int' object has no attribute 'matrix'
        """

        if other == 0:
            return self
        else:
            return _ScipyMatrix(matrix=self.matrix + other.matrix)

    __radd__ = __add__

    def __sub__(self, other):

        if other == 0:
            return self
        else:
            L = self.matrix.copy()
            L -= other.matrix
            return _ScipyMatrix(matrix=L)

    def __rsub__(self, other):
        return -self + other

    def __isub__(self, other):
        return self._iadd(other, sign=-1)

    def __mul__(self, other):
        """
        Multiply a sparse matrix by another sparse matrix

        >>> L1 = _ScipyMatrixFromShape(rows=3, cols=3)
        >>> L1.put([3., 10., numerix.pi, 2.5], [0, 0, 1, 2], [2, 1, 1, 0])
        >>> L2 = _ScipyIdentityMatrix(size=3)
        >>> L2.put([4.38, 12357.2, 1.1], [2, 1, 0], [1, 0, 2])

        >>> tmp = numerix.array(((1.23572000e+05, 2.31400000e+01, 3.00000000e+00),
        ...                      (3.88212887e+04, 3.14159265e+00, 0.00000000e+00),
        ...                      (2.50000000e+00, 0.00000000e+00, 2.75000000e+00)))

        >>> numerix.allclose((L1 * L2).numpyArray, tmp)
        1

        or a sparse matrix by a vector

        >>> tmp = numerix.array((29., 6.28318531, 2.5))
        >>> numerix.allclose(L1 * numerix.array((1, 2, 3), 'd'), tmp)
        1

        or a vector by a sparse matrix

        >>> tmp = numerix.array((7.5, 16.28318531,  3.))
        >>> numerix.allclose(numerix.array((1, 2, 3), 'd') * L1, tmp)
        1

        (The multiplication is broken.  Numpy is calling __rmul__ for every
        element instead of with the whole array.)
        """
        N = self.matrix.shape[0]

        if isinstance(other, _ScipyMatrix):
            return _ScipyMatrix(matrix=(self.matrix * other.matrix))
        else:
            shape = numerix.shape(other)
            if shape == ():
                return _ScipyMatrix(matrix=(self.matrix * other))
            elif shape == (N,):
                return self.matrix * other
            else:
                raise TypeError

    def __rmul__(self, other):
        if isinstance(numerix.ones(1, 'l'), type(other)):
            y = self.matrix.transpose() * other.copy()
            return y
        else:
            return self * other

    def asformat(self, *args, **kwargs):
        return self.matrix.asformat(*args, **kwargs)

    @property
    def _shape(self):
        return self.matrix.shape

    @property
    def _range(self):
        return list(range(self._shape[1])), list(range(self._shape[0]))

    def put(self, vector, id1, id2, overlapping=False):
        """Put elements of `vector` at positions of the matrix corresponding to (`id1`, `id2`)

        Parameters
        ----------
        vector : array_like
            The values to insert.
        id1 : array_like
            The row indices.
        id2 : array_like
            The column indices.
        overlapping : bool
            Whether to insert ghosted values or not (Ignored. Default False).

        Examples
        --------

            >>> L = _ScipyMatrixFromShape(rows=3, cols=3)
            >>> L.put([3., 10., numerix.pi, 2.5], [0, 0, 1, 2], [2, 1, 1, 0])
            >>> print(L)
                ---    10.000000   3.000000  
                ---     3.141593      ---    
             2.500000      ---        ---    
        """
        assert len(id1) == len(id2) == len(vector)

        # done in such a way to vectorize everything
        tempVec = numerix.array(vector) - self.matrix[id1, id2].flat
        tempMat = sp.csr_matrix((tempVec, (id1, id2)), self.matrix.shape)

        self.matrix = self.matrix + tempMat

    def putDiagonal(self, vector):
        """
        Put elements of `vector` along diagonal of matrix

            >>> L = _ScipyMatrixFromShape(rows=3, cols=3)
            >>> L.putDiagonal([3., 10., numerix.pi])
            >>> print(L)
             3.000000      ---        ---    
                ---    10.000000      ---    
                ---        ---     3.141593  
            >>> L.putDiagonal([10., 3.])
            >>> print(L)
            10.000000      ---        ---    
                ---     3.000000      ---    
                ---        ---     3.141593  
        """
        if isinstance(vector, (int, float)):
            vector = numerix.repeat(vector, self._shape[0])

        self.matrix.setdiag(vector)

    def take(self, id1, id2):
        return self.matrix[id1, id2]

    def takeDiagonal(self):
        return self.matrix.diagonal()

    def addAt(self, vector, id1, id2):
        """Add elements of `vector` to the positions in the matrix corresponding to (`id1`,`id2`)

        Parameters
        ----------
        vector : array_like
            The values to insert.
        id1 : array_like
            The row indices.
        id2 : array_like
            The column indices.
        overlapping : bool
            Whether to add ghosted values or not (Ignored. Default False).

        Examples
        --------

            >>> L = _ScipyMatrixFromShape(rows=3, cols=3)
            >>> L.put([3., 10., numerix.pi, 2.5], [0, 0, 1, 2], [2, 1, 1, 0])
            >>> L.addAt([1.73, 2.2, 8.4, 3.9, 1.23], [1, 2, 0, 0, 1], [2, 2, 0, 0, 2])
            >>> print(L)
            12.300000  10.000000   3.000000  
                ---     3.141593   2.960000  
             2.500000      ---     2.200000  
        """
        assert len(id1) == len(id2) == len(vector)

        temp = sp.csr_matrix((vector, (id1, id2)), self.matrix.shape)

        self.matrix = self.matrix + temp

    def addAtDiagonal(self, vector):
        if isinstance(vector, (int, float)):
            vector = numerix.repeat(vector, self._shape[0])

        ids = numerix.arange(len(vector))
        self.addAt(vector, ids, ids)

    @property
    def numpyArray(self):
        return self.matrix.toarray()

    def matvec(self, x):
        """This method is required for scipy solvers.
        """
        return self * x

    def exportMmf(self, filename):
        """Exports the matrix to a Matrix Market file of the given `filename`.
        """
        from scipy.io import mmio

        mmio.mmwrite(filename, self.matrix)

    @property
    def CSR(self):
        """The Compact Sparse Row description of the matrix

        Returns
        -------
        ptrs : array_like of int
            Locations in `cols` and `data` vectors that start a row,
            terminated with len(data) + 1
        cols : array_like of int
            Sequence of non-sparse column indices.
        data : array_like of float
            Sequence of non-sparse values.

        Examples
        --------

        >>> L = _ScipyMatrixFromShape(rows=3, cols=3, bandwidth=3)
        >>> L.put([3.,10.,numerix.pi,2.5], [0,0,1,2], [2,1,1,0])
        >>> L.addAt([1.73,2.2,8.4,3.9,1.23], [1,2,0,0,1], [2,2,0,0,2])
        >>> ptrs, cols, data = L.CSR
        >>> print(numerix.asarray(ptrs))
        [0 3 5 7]
        >>> print(numerix.asarray(cols))
        [0 1 2 1 2 0 2]
        >>> print(numerix.asarray(data))
        [ 12.3         10.           3.           3.14159265   2.96
           2.5          2.2       ]
        """
        return self.matrix.indptr, self.matrix.indices, self.matrix.data

    @property
    def LIL(self):
        """The List of Lists description of the local matrix

        Returns
        -------
        rows : list of sequence of int
            List of non-sparse column indices on each row.
        data : list of sequence of float
            List of non-sparse values on each row.

        Examples
        --------

        >>> L = _ScipyMatrixFromShape(rows=3, cols=3, bandwidth=3)
        >>> L.put([3.,10.,numerix.pi,2.5], [0,0,1,2], [2,1,1,0])
        >>> L.addAt([1.73,2.2,8.4,3.9,1.23], [1,2,0,0,1], [2,2,0,0,2])
        >>> rows, data = L.LIL
        >>> from scipy.stats.mstats import argstoarray # doctest: +SCIPY
        >>> print(argstoarray(*rows)) # doctest: +SCIPY
        [[0.0 1.0 2.0]
         [1.0 2.0 --]
         [0.0 2.0 --]]
        >>> print(argstoarray(*data)) # doctest: +SCIPY
        [[12.3 10.0 3.0]
         [3.141592653589793 2.96 --]
         [2.5 2.2 --]]
        """
        lil = self.matrix.tolil()

        return lil.rows, lil.data

    @property
    def T(self):
        """Transpose matrix

        Returns
        -------
        ~fipy.matrices.scipyMatrix._ScipyMatrix

        Examples
        --------

        >>> import fipy as fp

        >>> mesh = fp.Grid1D(nx=10)
        >>> ids = fp.CellVariable(mesh=mesh, value=mesh._globalOverlappingCellIDs)

        >>> mat = _ScipyColMeshMatrix(mesh=mesh, rows=1)
        >>> mat.put(vector=ids.value,
        ...         id1=[fp.parallelComm.procID] * mesh.numberOfCells,
        ...         id2=mesh._localOverlappingCellIDs,
        ...         overlapping=True)

        >>> print(mat.T.numpyArray)
        [[ 0.]
         [ 1.]
         [ 2.]
         [ 3.]
         [ 4.]
         [ 5.]
         [ 6.]
         [ 7.]
         [ 8.]
         [ 9.]]
        """
        return _ScipyMatrix(matrix=self.matrix.transpose(copy=True))

class _ScipyMatrixFromShape(_ScipyMatrix):

    def __init__(self, rows, cols, bandwidth=0, sizeHint=None, matrix=None, storeZeros=True):
        """Instantiates and wraps a scipy sparse matrix

        Parameters
        ----------
        mesh : ~fipy.meshes.mesh.Mesh
            The `Mesh` to assemble the matrix for.
        bandwidth : int
            The proposed band width of the matrix.
        sizeHint : int
            Estimate of the number of non-zeros
        matrix : ~scipy.sparse.csr_matrix
            Pre-assembled SciPy matrix to use for storage.
        storeZeros : bool
            Instructs scipy to store zero values if possible.
        """
        if matrix is None:
            matrix = sp.csr_matrix((rows, cols))

        super(_ScipyMatrixFromShape, self).__init__(matrix=matrix)

class _ScipyBaseMeshMatrix(_ScipyMatrixFromShape):
    def __init__(self, mesh, rows, cols, bandwidth=0, sizeHint=None,
                 matrix=None, storeZeros=True):
        """Creates a `_ScipyMatrixFromShape` associated with a `Mesh`.

        Parameters
        ----------
        mesh : ~fipy.meshes.mesh.Mesh
            The `Mesh` to assemble the matrix for.
        rows : int
            The number of local matrix rows.
        cols : int
            The number of local matrix columns.
        bandwidth : int
            The proposed band width of the matrix.
        sizeHint : int
            Estimate of the number of non-zeros.
        matrix : ~scipy.sparse.csr_matrix
            Pre-assembled SciPy matrix to use for storage.
        storeZeros : bool
            Instructs scipy to store zero values if possible.
        """
        self.mesh = mesh

        super(_ScipyBaseMeshMatrix, self).__init__(rows=rows,
                                                   cols=cols,
                                                   bandwidth=bandwidth,
                                                   sizeHint=sizeHint,
                                                   matrix=matrix,
                                                   storeZeros=storeZeros)

    def _getGhostedValues(self, var):
        """Obtain current ghost values from across processes

        Nothing to do for serial matrix.

        Returns
        -------
        ndarray
            Ghosted values
        """
        return var.value

class _ScipyRowMeshMatrix(_ScipyBaseMeshMatrix):
    def __init__(self, mesh, cols, numberOfEquations=1, bandwidth=0,
                 sizeHint=None, matrix=None, storeZeros=True):
        """Creates a `_ScipyBaseMeshMatrix` with rows associated with equations.

        Parameters
        ----------
        mesh : ~fipy.meshes.mesh.Mesh
            The `Mesh` to assemble the matrix for.
        cols : int
            The number of matrix columns.
        numberOfEquations : int
            The rows of the matrix are determined by
            `numberOfEquations * mesh.numberOfCells`.
        bandwidth : int
            The proposed band width of the matrix.
        sizeHint : int
            Estimate of the number of non-zeros
        matrix : ~scipy.sparse.csr_matrix
            Pre-assembled SciPy matrix to use for storage.
        storeZeros : bool
            Instructs scipy to store zero values if possible.
        """
        self.numberOfEquations = numberOfEquations

        super(_ScipyRowMeshMatrix, self).__init__(mesh=mesh,
                                                  rows=numberOfEquations * mesh.numberOfCells,
                                                  cols=cols,
                                                  bandwidth=bandwidth,
                                                  sizeHint=sizeHint,
                                                  matrix=matrix,
                                                  storeZeros=storeZeros)

class _ScipyColMeshMatrix(_ScipyBaseMeshMatrix):
    def __init__(self, mesh, rows, numberOfVariables=1, bandwidth=0,
                 sizeHint=None, matrix=None, storeZeros=True):
        """Creates a `_ScipyBaseMeshMatrix` with columns associated with solution variables.

        Parameters
        ----------
        mesh : ~fipy.meshes.mesh.Mesh
            The `Mesh` to assemble the matrix for.
        rows : int
            The number of matrix rows.
        numberOfVariables : int
            The columns of the matrix are determined by
            `numberOfVariables * mesh.globalNumberOfCells`.
        bandwidth : int
            The proposed band width of the matrix.
        sizeHint : int
            Estimate of the number of non-zeros.
        matrix : ~scipy.sparse.csr_matrix
            Pre-assembled SciPy matrix to use for storage.
        storeZeros : bool
            Instructs scipy to store zero values if possible.
        """
        self.numberOfVariables = numberOfVariables

        super(_ScipyColMeshMatrix, self).__init__(mesh=mesh,
                                                  rows=rows,
                                                  cols=numberOfVariables * mesh.numberOfCells,
                                                  bandwidth=bandwidth,
                                                  sizeHint=sizeHint,
                                                  matrix=matrix,
                                                  storeZeros=storeZeros)

class _ScipyMeshMatrix(_ScipyRowMeshMatrix):
    def __init__(self, mesh, numberOfVariables=1, numberOfEquations=1,
                 bandwidth=0, sizeHint=None, matrix=None, storeZeros=True):
        """Creates a `_ScipyBaseMeshMatrix` associated with equations and variables.

        Parameters
        ----------
        mesh : ~fipy.meshes.mesh.Mesh
            The `Mesh` to assemble the matrix for.
        numberOfVariables : int
            The columns of the matrix are determined by
            `numberOfVariables * mesh.numberOfCells`.
        numberOfEquations : int
            The rows of the matrix are determined by
            `numberOfEquations * mesh.numberOfCells`.
        bandwidth : int
            The proposed band width of the matrix.
        sizeHint : int
            Estimate of the number of non-zeros
        matrix : ~scipy.sparse.csr_matrix
            Pre-assembled SciPy matrix to use for storage.
        storeZeros : bool
            Instructs scipy to store zero values if possible.
        """
        self.numberOfVariables = numberOfVariables

        super(_ScipyMeshMatrix, self).__init__(mesh=mesh,
                                               cols=numberOfVariables * mesh.numberOfCells,
                                               bandwidth=bandwidth,
                                               sizeHint=sizeHint,
                                               matrix=matrix,
                                               numberOfEquations=numberOfEquations,
                                               storeZeros=storeZeros)

    def __mul__(self, other):
        if isinstance(other, _ScipyMeshMatrix):
            return _ScipyMeshMatrix(mesh=self.mesh,
                                    matrix=(self.matrix * other.matrix))
        else:
            return _ScipyMatrixFromShape.__mul__(self, other)

    def asTrilinosMeshMatrix(self):
        """Transforms a scipy matrix into a trilinos matrix and maintains the
        trilinos matrix as an attribute.

        Returns
        -------
        ~fipy.matrices.trilinosMatrix._TrilinosMatrix
        """
        raise NotImplementedError

    def flush(self):
        """Deletes the scipy matrix and calls `self.trilinosMatrix.flush()` if necessary.
        """
        if not getattr(self, 'cache', False):
            del self.matrix

    def _test(self):
        """
        Tests

        >>> m = _ScipyMatrixFromShape(rows=3, cols=3, storeZeros=True)
        >>> m.addAt((1., 0., 2.), (0, 2, 1), (1, 2, 0))
        >>> nonZeroIdx = m.matrix.nonzero()
        >>> print(not hasattr(m.matrix, 'storeZeros')
        ...       or numerix.allequal(nonZeroIdx, [(0, 1), (1, 0), (2, 2)]))
        True
        >>> print(not hasattr(m.matrix, 'storeZeros')
        ...       or numerix.allequal(m.matrix[nonZeroIdx].toarray(),
        ...                           [1., 2., 0.]))
        True
        >>> m = _ScipyMatrixFromShape(rows=3, cols=3, storeZeros=False)
        >>> m.addAt((1., 0., 2.), (0, 2, 1), (1, 2, 0))
        >>> nonZeroIdx = m.matrix.nonzero()
        >>> print(numerix.allequal(nonZeroIdx, [(0, 1), (1, 0)]))
        True
        >>> print(numerix.allequal(numerix.array(m.matrix[nonZeroIdx]), numerix.array([1.0, 2.0])))
        True

        """
        pass

class _ScipyIdentityMatrix(_ScipyMatrixFromShape):
    """
    Represents a sparse identity matrix for scipy.
    """
    def __init__(self, size):
        """
        Create a sparse matrix with `1` in the diagonal

            >>> print(_ScipyIdentityMatrix(size=3))
             1.000000      ---        ---    
                ---     1.000000      ---    
                ---        ---     1.000000  
        """
        _ScipyMatrixFromShape.__init__(self, rows=size, cols=size, bandwidth=1)
        ids = numerix.arange(size)
        self.put(numerix.ones(size, 'd'), ids, ids)

class _ScipyIdentityMeshMatrix(_ScipyIdentityMatrix):
    def __init__(self, mesh):
        """
        Create a sparse matrix associated with a `Mesh` with `1` in the diagonal

            >>> from fipy import Grid1D
            >>> from fipy.tools import serialComm
            >>> mesh = Grid1D(nx=3, communicator=serialComm)
            >>> print(_ScipyIdentityMeshMatrix(mesh=mesh))
             1.000000      ---        ---    
                ---     1.000000      ---    
                ---        ---     1.000000  
        """
        _ScipyIdentityMatrix.__init__(self, size=mesh.numberOfCells)

def _test():
    import fipy.tests.doctestPlus
    return fipy.tests.doctestPlus.testmod()

if __name__ == "__main__":
    _test()



from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

__docformat__ = 'restructuredtext'

from fipy.tools.numerix import random
from fipy.variables.noiseVariable import NoiseVariable

__all__ = ["GammaNoiseVariable"]
from future.utils import text_to_native_str
__all__ = [text_to_native_str(n) for n in __all__]

class GammaNoiseVariable(NoiseVariable):
    r"""
    Represents a gamma distribution of random numbers with the probability
    distribution

    .. math::

       x^{\alpha - 1}\frac{\beta^\alpha e^{-\beta x}}{\Gamma(\alpha)}

    with a shape parameter :math:`\alpha`, a rate parameter :math:`\beta`, and
    :math:`\Gamma(z) = \int_0^\infty t^{z - 1}e^{-t}\,dt`.

    Seed the random module for the sake of deterministic test results.

    >>> from fipy import numerix
    >>> numerix.random.seed(1)

    We generate noise on a uniform Cartesian mesh

    >>> from fipy.variables.variable import Variable
    >>> alpha = Variable()
    >>> beta = Variable()
    >>> from fipy.meshes import Grid2D
    >>> noise = GammaNoiseVariable(mesh = Grid2D(nx = 100, ny = 100), shape = alpha, rate = beta)

    We histogram the root-volume-weighted noise distribution

    >>> from fipy.variables.histogramVariable import HistogramVariable
    >>> histogram = HistogramVariable(distribution = noise, dx = 0.1, nx = 300)

    and compare to a Gaussian distribution

    >>> from fipy.variables.cellVariable import CellVariable
    >>> x = CellVariable(mesh=histogram.mesh, value=histogram.mesh.cellCenters[0])
    >>> from scipy.special import gamma as Gamma # doctest: +SCIPY
    >>> from fipy.tools.numerix import exp
    >>> gammadist = (x**(alpha - 1) * (beta**alpha * exp(-beta * x)) / Gamma(alpha)) # doctest: +SCIPY

    >>> if __name__ == '__main__':
    ...     from fipy import Viewer
    ...     viewer = Viewer(vars=noise, datamin=0, datamax=30)
    ...     histoplot = Viewer(vars=(histogram, gammadist),
    ...                        datamin=0, datamax=1)

    >>> from fipy.tools.numerix import arange

    >>> for shape in arange(1, 8, 1):
    ...     alpha.value = shape
    ...     for rate in arange(0.5, 2.5, 0.5):
    ...         beta.value = rate
    ...         if __name__ == '__main__':
    ...             import sys
    ...             print("alpha: %g, beta: %g" % (alpha, beta), file=sys.stderr)
    ...             viewer.plot()
    ...             histoplot.plot()

    >>> print(abs(noise.faceGrad.divergence.cellVolumeAverage) < 5e-15)
    1

    .. image:: fipy/variables/gamma.*
      :scale: 25
      :align: center
      :alt: random values with a gamma distribution

    .. image:: fipy/variables/gamma-histogram.*
      :scale: 25
      :align: center
      :alt: histogram of random values with a gamma distribution

    """
    def __init__(self, mesh, shape, rate, name = '', hasOld = 0):
        r"""
        Parameters
        ----------
        mesh : ~fipy.meshes.mesh.Mesh
            The mesh on which to define the noise.
        shape : float
            The shape parameter, :math:`\alpha`.
        rate : float
            The rate or inverse scale parameter, :math:`\beta`.

        """
        NoiseVariable.__init__(self, mesh = mesh, name = name, hasOld = hasOld)
        self.shapeParam = self._requires(shape)
        self.rate = self._requires(rate)

    def random(self):
        return random.gamma(shape=self.shapeParam, scale=self.rate,
                            size=[self.mesh.globalNumberOfCells])

def _test():
    import fipy.tests.doctestPlus
    return fipy.tests.doctestPlus.testmod()

if __name__ == "__main__":
    _test()


